# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import is_given, maybe_transform, strip_not_given, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.devices import package_list_params
from ...types.devices.package_list_response import PackageListResponse

__all__ = ["PackagesResource", "AsyncPackagesResource"]


class PackagesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PackagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#accessing-raw-response-data-eg-headers
        """
        return PackagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PackagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#with_streaming_response
        """
        return PackagesResourceWithStreamingResponse(self)

    def list(
        self,
        device_id: str,
        *,
        include_protected_packages: bool | Omit = omit,
        include_system_packages: bool | Omit = omit,
        x_device_display_id: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PackageListResponse]:
        """
        List packages

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not device_id:
            raise ValueError(f"Expected a non-empty value for `device_id` but received {device_id!r}")
        extra_headers = {
            **strip_not_given(
                {"X-Device-Display-ID": str(x_device_display_id) if is_given(x_device_display_id) else not_given}
            ),
            **(extra_headers or {}),
        }
        return self._get(
            f"/devices/{device_id}/packages",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include_protected_packages": include_protected_packages,
                        "include_system_packages": include_system_packages,
                    },
                    package_list_params.PackageListParams,
                ),
            ),
            cast_to=PackageListResponse,
        )


class AsyncPackagesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPackagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPackagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPackagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#with_streaming_response
        """
        return AsyncPackagesResourceWithStreamingResponse(self)

    async def list(
        self,
        device_id: str,
        *,
        include_protected_packages: bool | Omit = omit,
        include_system_packages: bool | Omit = omit,
        x_device_display_id: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PackageListResponse]:
        """
        List packages

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not device_id:
            raise ValueError(f"Expected a non-empty value for `device_id` but received {device_id!r}")
        extra_headers = {
            **strip_not_given(
                {"X-Device-Display-ID": str(x_device_display_id) if is_given(x_device_display_id) else not_given}
            ),
            **(extra_headers or {}),
        }
        return await self._get(
            f"/devices/{device_id}/packages",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "include_protected_packages": include_protected_packages,
                        "include_system_packages": include_system_packages,
                    },
                    package_list_params.PackageListParams,
                ),
            ),
            cast_to=PackageListResponse,
        )


class PackagesResourceWithRawResponse:
    def __init__(self, packages: PackagesResource) -> None:
        self._packages = packages

        self.list = to_raw_response_wrapper(
            packages.list,
        )


class AsyncPackagesResourceWithRawResponse:
    def __init__(self, packages: AsyncPackagesResource) -> None:
        self._packages = packages

        self.list = async_to_raw_response_wrapper(
            packages.list,
        )


class PackagesResourceWithStreamingResponse:
    def __init__(self, packages: PackagesResource) -> None:
        self._packages = packages

        self.list = to_streamed_response_wrapper(
            packages.list,
        )


class AsyncPackagesResourceWithStreamingResponse:
    def __init__(self, packages: AsyncPackagesResource) -> None:
        self._packages = packages

        self.list = async_to_streamed_response_wrapper(
            packages.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import TypeAlias

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["HookGetSampleDataResponse", "HookGetSampleDataResponseItem"]


class HookGetSampleDataResponseItem(BaseModel):
    """Sample webhook event data for testing/mapping in Zapier."""

    id: str
    """The subscription ID"""

    created_at: str = FieldInfo(alias="createdAt")
    """ISO timestamp of when the subscription was created"""

    events: List[str]
    """List of subscribed events"""

    state: str
    """The hook state"""

    url: str
    """The webhook URL"""

    updated_at: Optional[str] = FieldInfo(alias="updatedAt", default=None)
    """ISO timestamp of the last update"""


HookGetSampleDataResponse: TypeAlias = List[HookGetSampleDataResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import carrier_list_params, carrier_create_params, carrier_lookup_params, carrier_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.carrier import Carrier
from ..types.carrier_list_response import CarrierListResponse
from ..types.carrier_delete_response import CarrierDeleteResponse

__all__ = ["CarriersResource", "AsyncCarriersResource"]


class CarriersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CarriersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#accessing-raw-response-data-eg-headers
        """
        return CarriersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CarriersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#with_streaming_response
        """
        return CarriersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        country: str,
        mcc: str,
        mnc: str,
        operator: str,
        company: str | Omit = omit,
        country_code: str | Omit = omit,
        country_iso: str | Omit = omit,
        detail_url: str | Omit = omit,
        gsm_bands: str | Omit = omit,
        lte_bands: str | Omit = omit,
        mobile_prefix: str | Omit = omit,
        nsn_size: str | Omit = omit,
        number_format: str | Omit = omit,
        protocols: str | Omit = omit,
        umts_bands: str | Omit = omit,
        website: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Carrier:
        """
        Create a new carrier

        Args:
          country: Country name

          mcc: Mobile Country Code

          mnc: Mobile Network Code

          operator: Operator name

          company: Company name

          country_code: Country dialing code (e.g., +1)

          country_iso: ISO country code

          detail_url: URL to carrier details page

          gsm_bands: Supported GSM bands

          lte_bands: Supported LTE bands

          mobile_prefix: Mobile number prefix

          nsn_size: National Significant Number size

          number_format: Phone number format

          protocols: Supported protocols (comma-separated)

          umts_bands: Supported UMTS bands

          website: Company website

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/carriers",
            body=maybe_transform(
                {
                    "country": country,
                    "mcc": mcc,
                    "mnc": mnc,
                    "operator": operator,
                    "company": company,
                    "country_code": country_code,
                    "country_iso": country_iso,
                    "detail_url": detail_url,
                    "gsm_bands": gsm_bands,
                    "lte_bands": lte_bands,
                    "mobile_prefix": mobile_prefix,
                    "nsn_size": nsn_size,
                    "number_format": number_format,
                    "protocols": protocols,
                    "umts_bands": umts_bands,
                    "website": website,
                },
                carrier_create_params.CarrierCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Carrier,
        )

    def retrieve(
        self,
        carrier_id: int,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Carrier:
        """
        Get carrier by ID

        Args:
          carrier_id: Carrier ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            f"/carriers/{carrier_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Carrier,
        )

    def update(
        self,
        carrier_id: int,
        *,
        company: str | Omit = omit,
        country: str | Omit = omit,
        country_code: str | Omit = omit,
        country_iso: str | Omit = omit,
        detail_url: str | Omit = omit,
        gsm_bands: str | Omit = omit,
        lte_bands: str | Omit = omit,
        mobile_prefix: str | Omit = omit,
        nsn_size: str | Omit = omit,
        number_format: str | Omit = omit,
        operator: str | Omit = omit,
        protocols: str | Omit = omit,
        umts_bands: str | Omit = omit,
        website: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Carrier:
        """
        Update a carrier

        Args:
          carrier_id: Carrier ID

          company: Company name

          country: Country name

          country_code: Country dialing code

          country_iso: ISO country code

          detail_url: URL to carrier details

          gsm_bands: Supported GSM bands

          lte_bands: Supported LTE bands

          mobile_prefix: Mobile number prefix

          nsn_size: NSN size

          number_format: Phone number format

          operator: Operator name

          protocols: Supported protocols

          umts_bands: Supported UMTS bands

          website: Company website

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._patch(
            f"/carriers/{carrier_id}",
            body=maybe_transform(
                {
                    "company": company,
                    "country": country,
                    "country_code": country_code,
                    "country_iso": country_iso,
                    "detail_url": detail_url,
                    "gsm_bands": gsm_bands,
                    "lte_bands": lte_bands,
                    "mobile_prefix": mobile_prefix,
                    "nsn_size": nsn_size,
                    "number_format": number_format,
                    "operator": operator,
                    "protocols": protocols,
                    "umts_bands": umts_bands,
                    "website": website,
                },
                carrier_update_params.CarrierUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Carrier,
        )

    def list(
        self,
        *,
        country: str | Omit = omit,
        country_iso: str | Omit = omit,
        order_by: Literal["id", "mcc", "mnc", "operator", "country", "country_iso"] | Omit = omit,
        order_dir: Literal["asc", "desc"] | Omit = omit,
        page: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CarrierListResponse:
        """
        List carriers with pagination

        Args:
          country: Filter by country name

          country_iso: Filter by country ISO code

          order_by: Field to order by

          order_dir: Order direction

          page: Page number

          page_size: Items per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/carriers",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "country": country,
                        "country_iso": country_iso,
                        "order_by": order_by,
                        "order_dir": order_dir,
                        "page": page,
                        "page_size": page_size,
                    },
                    carrier_list_params.CarrierListParams,
                ),
            ),
            cast_to=CarrierListResponse,
        )

    def delete(
        self,
        carrier_id: int,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CarrierDeleteResponse:
        """
        Delete a carrier

        Args:
          carrier_id: Carrier ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._delete(
            f"/carriers/{carrier_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CarrierDeleteResponse,
        )

    def lookup(
        self,
        *,
        mcc: str,
        mnc: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Carrier:
        """
        Get carrier by MCC and MNC

        Args:
          mcc: Mobile Country Code

          mnc: Mobile Network Code

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/carriers/lookup",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "mcc": mcc,
                        "mnc": mnc,
                    },
                    carrier_lookup_params.CarrierLookupParams,
                ),
            ),
            cast_to=Carrier,
        )


class AsyncCarriersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCarriersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCarriersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCarriersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#with_streaming_response
        """
        return AsyncCarriersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        country: str,
        mcc: str,
        mnc: str,
        operator: str,
        company: str | Omit = omit,
        country_code: str | Omit = omit,
        country_iso: str | Omit = omit,
        detail_url: str | Omit = omit,
        gsm_bands: str | Omit = omit,
        lte_bands: str | Omit = omit,
        mobile_prefix: str | Omit = omit,
        nsn_size: str | Omit = omit,
        number_format: str | Omit = omit,
        protocols: str | Omit = omit,
        umts_bands: str | Omit = omit,
        website: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Carrier:
        """
        Create a new carrier

        Args:
          country: Country name

          mcc: Mobile Country Code

          mnc: Mobile Network Code

          operator: Operator name

          company: Company name

          country_code: Country dialing code (e.g., +1)

          country_iso: ISO country code

          detail_url: URL to carrier details page

          gsm_bands: Supported GSM bands

          lte_bands: Supported LTE bands

          mobile_prefix: Mobile number prefix

          nsn_size: National Significant Number size

          number_format: Phone number format

          protocols: Supported protocols (comma-separated)

          umts_bands: Supported UMTS bands

          website: Company website

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/carriers",
            body=await async_maybe_transform(
                {
                    "country": country,
                    "mcc": mcc,
                    "mnc": mnc,
                    "operator": operator,
                    "company": company,
                    "country_code": country_code,
                    "country_iso": country_iso,
                    "detail_url": detail_url,
                    "gsm_bands": gsm_bands,
                    "lte_bands": lte_bands,
                    "mobile_prefix": mobile_prefix,
                    "nsn_size": nsn_size,
                    "number_format": number_format,
                    "protocols": protocols,
                    "umts_bands": umts_bands,
                    "website": website,
                },
                carrier_create_params.CarrierCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Carrier,
        )

    async def retrieve(
        self,
        carrier_id: int,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Carrier:
        """
        Get carrier by ID

        Args:
          carrier_id: Carrier ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            f"/carriers/{carrier_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Carrier,
        )

    async def update(
        self,
        carrier_id: int,
        *,
        company: str | Omit = omit,
        country: str | Omit = omit,
        country_code: str | Omit = omit,
        country_iso: str | Omit = omit,
        detail_url: str | Omit = omit,
        gsm_bands: str | Omit = omit,
        lte_bands: str | Omit = omit,
        mobile_prefix: str | Omit = omit,
        nsn_size: str | Omit = omit,
        number_format: str | Omit = omit,
        operator: str | Omit = omit,
        protocols: str | Omit = omit,
        umts_bands: str | Omit = omit,
        website: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Carrier:
        """
        Update a carrier

        Args:
          carrier_id: Carrier ID

          company: Company name

          country: Country name

          country_code: Country dialing code

          country_iso: ISO country code

          detail_url: URL to carrier details

          gsm_bands: Supported GSM bands

          lte_bands: Supported LTE bands

          mobile_prefix: Mobile number prefix

          nsn_size: NSN size

          number_format: Phone number format

          operator: Operator name

          protocols: Supported protocols

          umts_bands: Supported UMTS bands

          website: Company website

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._patch(
            f"/carriers/{carrier_id}",
            body=await async_maybe_transform(
                {
                    "company": company,
                    "country": country,
                    "country_code": country_code,
                    "country_iso": country_iso,
                    "detail_url": detail_url,
                    "gsm_bands": gsm_bands,
                    "lte_bands": lte_bands,
                    "mobile_prefix": mobile_prefix,
                    "nsn_size": nsn_size,
                    "number_format": number_format,
                    "operator": operator,
                    "protocols": protocols,
                    "umts_bands": umts_bands,
                    "website": website,
                },
                carrier_update_params.CarrierUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Carrier,
        )

    async def list(
        self,
        *,
        country: str | Omit = omit,
        country_iso: str | Omit = omit,
        order_by: Literal["id", "mcc", "mnc", "operator", "country", "country_iso"] | Omit = omit,
        order_dir: Literal["asc", "desc"] | Omit = omit,
        page: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CarrierListResponse:
        """
        List carriers with pagination

        Args:
          country: Filter by country name

          country_iso: Filter by country ISO code

          order_by: Field to order by

          order_dir: Order direction

          page: Page number

          page_size: Items per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/carriers",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "country": country,
                        "country_iso": country_iso,
                        "order_by": order_by,
                        "order_dir": order_dir,
                        "page": page,
                        "page_size": page_size,
                    },
                    carrier_list_params.CarrierListParams,
                ),
            ),
            cast_to=CarrierListResponse,
        )

    async def delete(
        self,
        carrier_id: int,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CarrierDeleteResponse:
        """
        Delete a carrier

        Args:
          carrier_id: Carrier ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._delete(
            f"/carriers/{carrier_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CarrierDeleteResponse,
        )

    async def lookup(
        self,
        *,
        mcc: str,
        mnc: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Carrier:
        """
        Get carrier by MCC and MNC

        Args:
          mcc: Mobile Country Code

          mnc: Mobile Network Code

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/carriers/lookup",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "mcc": mcc,
                        "mnc": mnc,
                    },
                    carrier_lookup_params.CarrierLookupParams,
                ),
            ),
            cast_to=Carrier,
        )


class CarriersResourceWithRawResponse:
    def __init__(self, carriers: CarriersResource) -> None:
        self._carriers = carriers

        self.create = to_raw_response_wrapper(
            carriers.create,
        )
        self.retrieve = to_raw_response_wrapper(
            carriers.retrieve,
        )
        self.update = to_raw_response_wrapper(
            carriers.update,
        )
        self.list = to_raw_response_wrapper(
            carriers.list,
        )
        self.delete = to_raw_response_wrapper(
            carriers.delete,
        )
        self.lookup = to_raw_response_wrapper(
            carriers.lookup,
        )


class AsyncCarriersResourceWithRawResponse:
    def __init__(self, carriers: AsyncCarriersResource) -> None:
        self._carriers = carriers

        self.create = async_to_raw_response_wrapper(
            carriers.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            carriers.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            carriers.update,
        )
        self.list = async_to_raw_response_wrapper(
            carriers.list,
        )
        self.delete = async_to_raw_response_wrapper(
            carriers.delete,
        )
        self.lookup = async_to_raw_response_wrapper(
            carriers.lookup,
        )


class CarriersResourceWithStreamingResponse:
    def __init__(self, carriers: CarriersResource) -> None:
        self._carriers = carriers

        self.create = to_streamed_response_wrapper(
            carriers.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            carriers.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            carriers.update,
        )
        self.list = to_streamed_response_wrapper(
            carriers.list,
        )
        self.delete = to_streamed_response_wrapper(
            carriers.delete,
        )
        self.lookup = to_streamed_response_wrapper(
            carriers.lookup,
        )


class AsyncCarriersResourceWithStreamingResponse:
    def __init__(self, carriers: AsyncCarriersResource) -> None:
        self._carriers = carriers

        self.create = async_to_streamed_response_wrapper(
            carriers.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            carriers.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            carriers.update,
        )
        self.list = async_to_streamed_response_wrapper(
            carriers.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            carriers.delete,
        )
        self.lookup = async_to_streamed_response_wrapper(
            carriers.lookup,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .meta import Meta as Meta
from .config import Config as Config
from .pagination import Pagination as Pagination
from .device_spec import DeviceSpec as DeviceSpec
from .device_carrier import DeviceCarrier as DeviceCarrier
from .permission_set import PermissionSet as PermissionSet
from .pagination_meta import PaginationMeta as PaginationMeta
from .device_identifiers import DeviceIdentifiers as DeviceIdentifiers



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from mobilerun import Mobilerun, AsyncMobilerun
from tests.utils import assert_matches_type
from mobilerun.types.devices import (
    ActionOverlayVisibleResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestActions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_global(self, client: Mobilerun) -> None:
        action = client.devices.actions.global_(
            device_id="deviceId",
            action=0,
        )
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_global_with_all_params(self, client: Mobilerun) -> None:
        action = client.devices.actions.global_(
            device_id="deviceId",
            action=0,
            x_device_display_id=0,
        )
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_global(self, client: Mobilerun) -> None:
        response = client.devices.actions.with_raw_response.global_(
            device_id="deviceId",
            action=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = response.parse()
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_global(self, client: Mobilerun) -> None:
        with client.devices.actions.with_streaming_response.global_(
            device_id="deviceId",
            action=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = response.parse()
            assert action is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_global(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            client.devices.actions.with_raw_response.global_(
                device_id="",
                action=0,
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_overlay_visible(self, client: Mobilerun) -> None:
        action = client.devices.actions.overlay_visible(
            device_id="deviceId",
        )
        assert_matches_type(ActionOverlayVisibleResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_overlay_visible_with_all_params(self, client: Mobilerun) -> None:
        action = client.devices.actions.overlay_visible(
            device_id="deviceId",
            x_device_display_id=0,
        )
        assert_matches_type(ActionOverlayVisibleResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_overlay_visible(self, client: Mobilerun) -> None:
        response = client.devices.actions.with_raw_response.overlay_visible(
            device_id="deviceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = response.parse()
        assert_matches_type(ActionOverlayVisibleResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_overlay_visible(self, client: Mobilerun) -> None:
        with client.devices.actions.with_streaming_response.overlay_visible(
            device_id="deviceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = response.parse()
            assert_matches_type(ActionOverlayVisibleResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_overlay_visible(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            client.devices.actions.with_raw_response.overlay_visible(
                device_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_set_overlay_visible(self, client: Mobilerun) -> None:
        action = client.devices.actions.set_overlay_visible(
            device_id="deviceId",
            visible=True,
        )
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_set_overlay_visible_with_all_params(self, client: Mobilerun) -> None:
        action = client.devices.actions.set_overlay_visible(
            device_id="deviceId",
            visible=True,
            x_device_display_id=0,
        )
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_set_overlay_visible(self, client: Mobilerun) -> None:
        response = client.devices.actions.with_raw_response.set_overlay_visible(
            device_id="deviceId",
            visible=True,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = response.parse()
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_set_overlay_visible(self, client: Mobilerun) -> None:
        with client.devices.actions.with_streaming_response.set_overlay_visible(
            device_id="deviceId",
            visible=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = response.parse()
            assert action is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_set_overlay_visible(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            client.devices.actions.with_raw_response.set_overlay_visible(
                device_id="",
                visible=True,
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_swipe(self, client: Mobilerun) -> None:
        action = client.devices.actions.swipe(
            device_id="deviceId",
            duration=10,
            end_x=0,
            end_y=0,
            start_x=0,
            start_y=0,
        )
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_swipe_with_all_params(self, client: Mobilerun) -> None:
        action = client.devices.actions.swipe(
            device_id="deviceId",
            duration=10,
            end_x=0,
            end_y=0,
            start_x=0,
            start_y=0,
            stealth=True,
            x_device_display_id=0,
        )
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_swipe(self, client: Mobilerun) -> None:
        response = client.devices.actions.with_raw_response.swipe(
            device_id="deviceId",
            duration=10,
            end_x=0,
            end_y=0,
            start_x=0,
            start_y=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = response.parse()
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_swipe(self, client: Mobilerun) -> None:
        with client.devices.actions.with_streaming_response.swipe(
            device_id="deviceId",
            duration=10,
            end_x=0,
            end_y=0,
            start_x=0,
            start_y=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = response.parse()
            assert action is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_swipe(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            client.devices.actions.with_raw_response.swipe(
                device_id="",
                duration=10,
                end_x=0,
                end_y=0,
                start_x=0,
                start_y=0,
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_tap(self, client: Mobilerun) -> None:
        action = client.devices.actions.tap(
            device_id="deviceId",
            x=0,
            y=0,
        )
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_tap_with_all_params(self, client: Mobilerun) -> None:
        action = client.devices.actions.tap(
            device_id="deviceId",
            x=0,
            y=0,
            stealth=True,
            x_device_display_id=0,
        )
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_tap(self, client: Mobilerun) -> None:
        response = client.devices.actions.with_raw_response.tap(
            device_id="deviceId",
            x=0,
            y=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = response.parse()
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_tap(self, client: Mobilerun) -> None:
        with client.devices.actions.with_streaming_response.tap(
            device_id="deviceId",
            x=0,
            y=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = response.parse()
            assert action is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_tap(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            client.devices.actions.with_raw_response.tap(
                device_id="",
                x=0,
                y=0,
            )


class TestAsyncActions:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_global(self, async_client: AsyncMobilerun) -> None:
        action = await async_client.devices.actions.global_(
            device_id="deviceId",
            action=0,
        )
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_global_with_all_params(self, async_client: AsyncMobilerun) -> None:
        action = await async_client.devices.actions.global_(
            device_id="deviceId",
            action=0,
            x_device_display_id=0,
        )
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_global(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.devices.actions.with_raw_response.global_(
            device_id="deviceId",
            action=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = await response.parse()
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_global(self, async_client: AsyncMobilerun) -> None:
        async with async_client.devices.actions.with_streaming_response.global_(
            device_id="deviceId",
            action=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = await response.parse()
            assert action is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_global(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            await async_client.devices.actions.with_raw_response.global_(
                device_id="",
                action=0,
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_overlay_visible(self, async_client: AsyncMobilerun) -> None:
        action = await async_client.devices.actions.overlay_visible(
            device_id="deviceId",
        )
        assert_matches_type(ActionOverlayVisibleResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_overlay_visible_with_all_params(self, async_client: AsyncMobilerun) -> None:
        action = await async_client.devices.actions.overlay_visible(
            device_id="deviceId",
            x_device_display_id=0,
        )
        assert_matches_type(ActionOverlayVisibleResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_overlay_visible(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.devices.actions.with_raw_response.overlay_visible(
            device_id="deviceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = await response.parse()
        assert_matches_type(ActionOverlayVisibleResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_overlay_visible(self, async_client: AsyncMobilerun) -> None:
        async with async_client.devices.actions.with_streaming_response.overlay_visible(
            device_id="deviceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = await response.parse()
            assert_matches_type(ActionOverlayVisibleResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_overlay_visible(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            await async_client.devices.actions.with_raw_response.overlay_visible(
                device_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_set_overlay_visible(self, async_client: AsyncMobilerun) -> None:
        action = await async_client.devices.actions.set_overlay_visible(
            device_id="deviceId",
            visible=True,
        )
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_set_overlay_visible_with_all_params(self, async_client: AsyncMobilerun) -> None:
        action = await async_client.devices.actions.set_overlay_visible(
            device_id="deviceId",
            visible=True,
            x_device_display_id=0,
        )
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_set_overlay_visible(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.devices.actions.with_raw_response.set_overlay_visible(
            device_id="deviceId",
            visible=True,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = await response.parse()
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_set_overlay_visible(self, async_client: AsyncMobilerun) -> None:
        async with async_client.devices.actions.with_streaming_response.set_overlay_visible(
            device_id="deviceId",
            visible=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = await response.parse()
            assert action is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_set_overlay_visible(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            await async_client.devices.actions.with_raw_response.set_overlay_visible(
                device_id="",
                visible=True,
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_swipe(self, async_client: AsyncMobilerun) -> None:
        action = await async_client.devices.actions.swipe(
            device_id="deviceId",
            duration=10,
            end_x=0,
            end_y=0,
            start_x=0,
            start_y=0,
        )
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_swipe_with_all_params(self, async_client: AsyncMobilerun) -> None:
        action = await async_client.devices.actions.swipe(
            device_id="deviceId",
            duration=10,
            end_x=0,
            end_y=0,
            start_x=0,
            start_y=0,
            stealth=True,
            x_device_display_id=0,
        )
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_swipe(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.devices.actions.with_raw_response.swipe(
            device_id="deviceId",
            duration=10,
            end_x=0,
            end_y=0,
            start_x=0,
            start_y=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = await response.parse()
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_swipe(self, async_client: AsyncMobilerun) -> None:
        async with async_client.devices.actions.with_streaming_response.swipe(
            device_id="deviceId",
            duration=10,
            end_x=0,
            end_y=0,
            start_x=0,
            start_y=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = await response.parse()
            assert action is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_swipe(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            await async_client.devices.actions.with_raw_response.swipe(
                device_id="",
                duration=10,
                end_x=0,
                end_y=0,
                start_x=0,
                start_y=0,
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_tap(self, async_client: AsyncMobilerun) -> None:
        action = await async_client.devices.actions.tap(
            device_id="deviceId",
            x=0,
            y=0,
        )
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_tap_with_all_params(self, async_client: AsyncMobilerun) -> None:
        action = await async_client.devices.actions.tap(
            device_id="deviceId",
            x=0,
            y=0,
            stealth=True,
            x_device_display_id=0,
        )
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_tap(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.devices.actions.with_raw_response.tap(
            device_id="deviceId",
            x=0,
            y=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = await response.parse()
        assert action is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_tap(self, async_client: AsyncMobilerun) -> None:
        async with async_client.devices.actions.with_streaming_response.tap(
            device_id="deviceId",
            x=0,
            y=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = await response.parse()
            assert action is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_tap(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            await async_client.devices.actions.with_raw_response.tap(
                device_id="",
                x=0,
                y=0,
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ActionSwipeParams"]


class ActionSwipeParams(TypedDict, total=False):
    duration: Required[int]
    """Swipe duration in milliseconds"""

    end_x: Required[Annotated[int, PropertyInfo(alias="endX")]]

    end_y: Required[Annotated[int, PropertyInfo(alias="endY")]]

    start_x: Required[Annotated[int, PropertyInfo(alias="startX")]]

    start_y: Required[Annotated[int, PropertyInfo(alias="startY")]]

    stealth: bool

    x_device_display_id: Annotated[int, PropertyInfo(alias="X-Device-Display-ID")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from ..shared.meta import Meta

__all__ = ["TaskListResponse", "Item"]


class Item(BaseModel):
    created_at: datetime = FieldInfo(alias="createdAt")

    task_id: str = FieldInfo(alias="taskId")

    updated_at: datetime = FieldInfo(alias="updatedAt")


class TaskListResponse(BaseModel):
    items: Optional[List[Item]] = None

    pagination: Meta

    schema_: Optional[str] = FieldInfo(alias="$schema", default=None)
    """A URL to the JSON Schema for this object."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable, Optional
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo
from .package_credentials_param import PackageCredentialsParam

__all__ = ["TaskRunParams"]


class TaskRunParams(TypedDict, total=False):
    device_id: Required[Annotated[str, PropertyInfo(alias="deviceId")]]
    """The ID of the device to run the task on."""

    task: Required[str]

    agent_id: Annotated[int, PropertyInfo(alias="agentId")]

    apps: SequenceNotStr[str]

    credentials: Iterable[PackageCredentialsParam]

    display_id: Annotated[int, PropertyInfo(alias="displayId")]
    """The display ID of the device to run the task on."""

    execution_timeout: Annotated[int, PropertyInfo(alias="executionTimeout")]

    files: SequenceNotStr[str]

    llm_model: Annotated[str, PropertyInfo(alias="llmModel")]
    """The LLM model identifier to use for the task (e.g.

    'google/gemini-3.1-flash-lite-preview')
    """

    max_steps: Annotated[int, PropertyInfo(alias="maxSteps")]

    output_schema: Annotated[Optional[Dict[str, object]], PropertyInfo(alias="outputSchema")]

    reasoning: bool

    stealth: bool

    temperature: float

    vision: bool

    vpn_country: Annotated[
        Optional[Literal["US", "BR", "FR", "DE", "IN", "JP", "KR", "ZA"]], PropertyInfo(alias="vpnCountry")
    ]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel
from .proxy_config import ProxyConfig

__all__ = ["ProxyListResponse"]


class ProxyListResponse(BaseModel):
    data: List[ProxyConfig]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ActionTapParams"]


class ActionTapParams(TypedDict, total=False):
    x: Required[int]

    y: Required[int]

    stealth: bool

    x_device_display_id: Annotated[int, PropertyInfo(alias="X-Device-Display-ID")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["TaskStopResponse"]


class TaskStopResponse(BaseModel):
    cancelled: bool
    """Whether the task was cancelled"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .rect import Rect
from ..._models import BaseModel

__all__ = [
    "StateUiResponse",
    "DeviceContext",
    "DeviceContextDisplayMetrics",
    "DeviceContextFilteringParams",
    "PhoneState",
    "PhoneStateFocusedElement",
]


class DeviceContextDisplayMetrics(BaseModel):
    density: float

    density_dpi: int = FieldInfo(alias="densityDpi")

    height_pixels: int = FieldInfo(alias="heightPixels")

    scaled_density: float = FieldInfo(alias="scaledDensity")

    width_pixels: int = FieldInfo(alias="widthPixels")


class DeviceContextFilteringParams(BaseModel):
    min_element_size: int

    overlay_offset: int


class DeviceContext(BaseModel):
    display_metrics: DeviceContextDisplayMetrics

    filtering_params: DeviceContextFilteringParams

    screen_bounds: Rect

    screen_size: Rect = FieldInfo(alias="screenSize")


class PhoneStateFocusedElement(BaseModel):
    class_name: Optional[str] = FieldInfo(alias="className", default=None)

    resource_id: Optional[str] = FieldInfo(alias="resourceId", default=None)

    text: Optional[str] = None


class PhoneState(BaseModel):
    is_editable: bool = FieldInfo(alias="isEditable")

    keyboard_visible: bool = FieldInfo(alias="keyboardVisible")

    activity_name: Optional[str] = FieldInfo(alias="activityName", default=None)

    current_app: Optional[str] = FieldInfo(alias="currentApp", default=None)

    focused_element: Optional[PhoneStateFocusedElement] = FieldInfo(alias="focusedElement", default=None)

    package_name: Optional[str] = FieldInfo(alias="packageName", default=None)


class StateUiResponse(BaseModel):
    a11y_tree: object

    device_context: DeviceContext

    phone_state: PhoneState

    schema_: Optional[str] = FieldInfo(alias="$schema", default=None)
    """A URL to the JSON Schema for this object."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...types import credential_list_params
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from .packages.packages import (
    PackagesResource,
    AsyncPackagesResource,
    PackagesResourceWithRawResponse,
    AsyncPackagesResourceWithRawResponse,
    PackagesResourceWithStreamingResponse,
    AsyncPackagesResourceWithStreamingResponse,
)
from ...types.credential_list_response import CredentialListResponse

__all__ = ["CredentialsResource", "AsyncCredentialsResource"]


class CredentialsResource(SyncAPIResource):
    @cached_property
    def packages(self) -> PackagesResource:
        return PackagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> CredentialsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#accessing-raw-response-data-eg-headers
        """
        return CredentialsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CredentialsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#with_streaming_response
        """
        return CredentialsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        page: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CredentialListResponse:
        """
        List all credentials for the authenticated user

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/credentials",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "page_size": page_size,
                    },
                    credential_list_params.CredentialListParams,
                ),
            ),
            cast_to=CredentialListResponse,
        )


class AsyncCredentialsResource(AsyncAPIResource):
    @cached_property
    def packages(self) -> AsyncPackagesResource:
        return AsyncPackagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncCredentialsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCredentialsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCredentialsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/droidrun/mobilerun-sdk-python#with_streaming_response
        """
        return AsyncCredentialsResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        page: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CredentialListResponse:
        """
        List all credentials for the authenticated user

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/credentials",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "page": page,
                        "page_size": page_size,
                    },
                    credential_list_params.CredentialListParams,
                ),
            ),
            cast_to=CredentialListResponse,
        )


class CredentialsResourceWithRawResponse:
    def __init__(self, credentials: CredentialsResource) -> None:
        self._credentials = credentials

        self.list = to_raw_response_wrapper(
            credentials.list,
        )

    @cached_property
    def packages(self) -> PackagesResourceWithRawResponse:
        return PackagesResourceWithRawResponse(self._credentials.packages)


class AsyncCredentialsResourceWithRawResponse:
    def __init__(self, credentials: AsyncCredentialsResource) -> None:
        self._credentials = credentials

        self.list = async_to_raw_response_wrapper(
            credentials.list,
        )

    @cached_property
    def packages(self) -> AsyncPackagesResourceWithRawResponse:
        return AsyncPackagesResourceWithRawResponse(self._credentials.packages)


class CredentialsResourceWithStreamingResponse:
    def __init__(self, credentials: CredentialsResource) -> None:
        self._credentials = credentials

        self.list = to_streamed_response_wrapper(
            credentials.list,
        )

    @cached_property
    def packages(self) -> PackagesResourceWithStreamingResponse:
        return PackagesResourceWithStreamingResponse(self._credentials.packages)


class AsyncCredentialsResourceWithStreamingResponse:
    def __init__(self, credentials: AsyncCredentialsResource) -> None:
        self._credentials = credentials

        self.list = async_to_streamed_response_wrapper(
            credentials.list,
        )

    @cached_property
    def packages(self) -> AsyncPackagesResourceWithStreamingResponse:
        return AsyncPackagesResourceWithStreamingResponse(self._credentials.packages)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from .._models import BaseModel
from .proxy_config import ProxyConfig

__all__ = ["ProxyCreateResponse"]


class ProxyCreateResponse(BaseModel):
    data: ProxyConfig

    message: str

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ....._utils import PropertyInfo

__all__ = ["FieldUpdateParams"]


class FieldUpdateParams(TypedDict, total=False):
    package_name: Required[Annotated[str, PropertyInfo(alias="packageName")]]

    credential_name: Required[Annotated[str, PropertyInfo(alias="credentialName")]]

    value: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["TaskRunResponse"]


class TaskRunResponse(BaseModel):
    id: str
    """The ID of the task"""

    stream_url: str = FieldInfo(alias="streamUrl")
    """The URL of the stream"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from mobilerun import Mobilerun, AsyncMobilerun
from tests.utils import assert_matches_type
from mobilerun.types import (
    Profile,
    ProfileListResponse,
    ProfileDeleteResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestProfiles:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Mobilerun) -> None:
        profile = client.profiles.create(
            name="x",
            spec={},
        )
        assert_matches_type(Profile, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Mobilerun) -> None:
        profile = client.profiles.create(
            name="x",
            spec={
                "apps": ["string"],
                "carrier": {
                    "gsm_operator_alpha": "GsmOperatorAlpha",
                    "gsm_operator_numeric": 0,
                    "gsm_sim_operator_alpha": "GsmSimOperatorAlpha",
                    "gsm_sim_operator_iso_country": "GsmSimOperatorIsoCountry",
                    "gsm_sim_operator_numeric": 0,
                    "persist_sys_timezone": "PersistSysTimezone",
                },
                "files": ["string"],
                "identifiers": {
                    "bootloader_serial_number": "BootloaderSerialNumber",
                    "identifier_android_id": "IdentifierAndroidID",
                    "identifier_app_set_id": "IdentifierAppSetID",
                    "identifier_bluetooth_mac": "IdentifierBluetoothMAC",
                    "identifier_gaid": "IdentifierGAID",
                    "identifier_gsfid": "IdentifierGSFID",
                    "identifier_iccid": "IdentifierICCID",
                    "identifier_imei": "IdentifierIMEI",
                    "identifier_imsi": "IdentifierIMSI",
                    "identifier_media_drmid": "IdentifierMediaDRMID",
                    "identifier_meid": "IdentifierMEID",
                    "identifier_phone_number": "IdentifierPhoneNumber",
                    "identifier_serial": "IdentifierSerial",
                    "identifier_wifi_mac": "IdentifierWifiMAC",
                    "serial_number": "SerialNumber",
                },
                "name": "name",
                "proxy": {
                    "host": "host",
                    "password": "password",
                    "port": 0,
                    "user": "user",
                },
                "smart_ip": True,
            },
        )
        assert_matches_type(Profile, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Mobilerun) -> None:
        response = client.profiles.with_raw_response.create(
            name="x",
            spec={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        profile = response.parse()
        assert_matches_type(Profile, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Mobilerun) -> None:
        with client.profiles.with_streaming_response.create(
            name="x",
            spec={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            profile = response.parse()
            assert_matches_type(Profile, profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Mobilerun) -> None:
        profile = client.profiles.retrieve(
            "profileId",
        )
        assert_matches_type(Profile, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Mobilerun) -> None:
        response = client.profiles.with_raw_response.retrieve(
            "profileId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        profile = response.parse()
        assert_matches_type(Profile, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Mobilerun) -> None:
        with client.profiles.with_streaming_response.retrieve(
            "profileId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            profile = response.parse()
            assert_matches_type(Profile, profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `profile_id` but received ''"):
            client.profiles.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Mobilerun) -> None:
        profile = client.profiles.update(
            profile_id="profileId",
            name="x",
            spec={},
        )
        assert_matches_type(Profile, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Mobilerun) -> None:
        profile = client.profiles.update(
            profile_id="profileId",
            name="x",
            spec={
                "apps": ["string"],
                "carrier": {
                    "gsm_operator_alpha": "GsmOperatorAlpha",
                    "gsm_operator_numeric": 0,
                    "gsm_sim_operator_alpha": "GsmSimOperatorAlpha",
                    "gsm_sim_operator_iso_country": "GsmSimOperatorIsoCountry",
                    "gsm_sim_operator_numeric": 0,
                    "persist_sys_timezone": "PersistSysTimezone",
                },
                "files": ["string"],
                "identifiers": {
                    "bootloader_serial_number": "BootloaderSerialNumber",
                    "identifier_android_id": "IdentifierAndroidID",
                    "identifier_app_set_id": "IdentifierAppSetID",
                    "identifier_bluetooth_mac": "IdentifierBluetoothMAC",
                    "identifier_gaid": "IdentifierGAID",
                    "identifier_gsfid": "IdentifierGSFID",
                    "identifier_iccid": "IdentifierICCID",
                    "identifier_imei": "IdentifierIMEI",
                    "identifier_imsi": "IdentifierIMSI",
                    "identifier_media_drmid": "IdentifierMediaDRMID",
                    "identifier_meid": "IdentifierMEID",
                    "identifier_phone_number": "IdentifierPhoneNumber",
                    "identifier_serial": "IdentifierSerial",
                    "identifier_wifi_mac": "IdentifierWifiMAC",
                    "serial_number": "SerialNumber",
                },
                "name": "name",
                "proxy": {
                    "host": "host",
                    "password": "password",
                    "port": 0,
                    "user": "user",
                },
                "smart_ip": True,
            },
        )
        assert_matches_type(Profile, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Mobilerun) -> None:
        response = client.profiles.with_raw_response.update(
            profile_id="profileId",
            name="x",
            spec={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        profile = response.parse()
        assert_matches_type(Profile, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Mobilerun) -> None:
        with client.profiles.with_streaming_response.update(
            profile_id="profileId",
            name="x",
            spec={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            profile = response.parse()
            assert_matches_type(Profile, profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `profile_id` but received ''"):
            client.profiles.with_raw_response.update(
                profile_id="",
                name="x",
                spec={},
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Mobilerun) -> None:
        profile = client.profiles.list()
        assert_matches_type(ProfileListResponse, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Mobilerun) -> None:
        profile = client.profiles.list(
            name="name",
            order_by="name",
            order_by_direction="asc",
            page=0,
            page_size=0,
        )
        assert_matches_type(ProfileListResponse, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Mobilerun) -> None:
        response = client.profiles.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        profile = response.parse()
        assert_matches_type(ProfileListResponse, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Mobilerun) -> None:
        with client.profiles.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            profile = response.parse()
            assert_matches_type(ProfileListResponse, profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Mobilerun) -> None:
        profile = client.profiles.delete(
            "profileId",
        )
        assert_matches_type(ProfileDeleteResponse, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Mobilerun) -> None:
        response = client.profiles.with_raw_response.delete(
            "profileId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        profile = response.parse()
        assert_matches_type(ProfileDeleteResponse, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Mobilerun) -> None:
        with client.profiles.with_streaming_response.delete(
            "profileId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            profile = response.parse()
            assert_matches_type(ProfileDeleteResponse, profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `profile_id` but received ''"):
            client.profiles.with_raw_response.delete(
                "",
            )


class TestAsyncProfiles:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncMobilerun) -> None:
        profile = await async_client.profiles.create(
            name="x",
            spec={},
        )
        assert_matches_type(Profile, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncMobilerun) -> None:
        profile = await async_client.profiles.create(
            name="x",
            spec={
                "apps": ["string"],
                "carrier": {
                    "gsm_operator_alpha": "GsmOperatorAlpha",
                    "gsm_operator_numeric": 0,
                    "gsm_sim_operator_alpha": "GsmSimOperatorAlpha",
                    "gsm_sim_operator_iso_country": "GsmSimOperatorIsoCountry",
                    "gsm_sim_operator_numeric": 0,
                    "persist_sys_timezone": "PersistSysTimezone",
                },
                "files": ["string"],
                "identifiers": {
                    "bootloader_serial_number": "BootloaderSerialNumber",
                    "identifier_android_id": "IdentifierAndroidID",
                    "identifier_app_set_id": "IdentifierAppSetID",
                    "identifier_bluetooth_mac": "IdentifierBluetoothMAC",
                    "identifier_gaid": "IdentifierGAID",
                    "identifier_gsfid": "IdentifierGSFID",
                    "identifier_iccid": "IdentifierICCID",
                    "identifier_imei": "IdentifierIMEI",
                    "identifier_imsi": "IdentifierIMSI",
                    "identifier_media_drmid": "IdentifierMediaDRMID",
                    "identifier_meid": "IdentifierMEID",
                    "identifier_phone_number": "IdentifierPhoneNumber",
                    "identifier_serial": "IdentifierSerial",
                    "identifier_wifi_mac": "IdentifierWifiMAC",
                    "serial_number": "SerialNumber",
                },
                "name": "name",
                "proxy": {
                    "host": "host",
                    "password": "password",
                    "port": 0,
                    "user": "user",
                },
                "smart_ip": True,
            },
        )
        assert_matches_type(Profile, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.profiles.with_raw_response.create(
            name="x",
            spec={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        profile = await response.parse()
        assert_matches_type(Profile, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncMobilerun) -> None:
        async with async_client.profiles.with_streaming_response.create(
            name="x",
            spec={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            profile = await response.parse()
            assert_matches_type(Profile, profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncMobilerun) -> None:
        profile = await async_client.profiles.retrieve(
            "profileId",
        )
        assert_matches_type(Profile, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.profiles.with_raw_response.retrieve(
            "profileId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        profile = await response.parse()
        assert_matches_type(Profile, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncMobilerun) -> None:
        async with async_client.profiles.with_streaming_response.retrieve(
            "profileId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            profile = await response.parse()
            assert_matches_type(Profile, profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `profile_id` but received ''"):
            await async_client.profiles.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncMobilerun) -> None:
        profile = await async_client.profiles.update(
            profile_id="profileId",
            name="x",
            spec={},
        )
        assert_matches_type(Profile, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncMobilerun) -> None:
        profile = await async_client.profiles.update(
            profile_id="profileId",
            name="x",
            spec={
                "apps": ["string"],
                "carrier": {
                    "gsm_operator_alpha": "GsmOperatorAlpha",
                    "gsm_operator_numeric": 0,
                    "gsm_sim_operator_alpha": "GsmSimOperatorAlpha",
                    "gsm_sim_operator_iso_country": "GsmSimOperatorIsoCountry",
                    "gsm_sim_operator_numeric": 0,
                    "persist_sys_timezone": "PersistSysTimezone",
                },
                "files": ["string"],
                "identifiers": {
                    "bootloader_serial_number": "BootloaderSerialNumber",
                    "identifier_android_id": "IdentifierAndroidID",
                    "identifier_app_set_id": "IdentifierAppSetID",
                    "identifier_bluetooth_mac": "IdentifierBluetoothMAC",
                    "identifier_gaid": "IdentifierGAID",
                    "identifier_gsfid": "IdentifierGSFID",
                    "identifier_iccid": "IdentifierICCID",
                    "identifier_imei": "IdentifierIMEI",
                    "identifier_imsi": "IdentifierIMSI",
                    "identifier_media_drmid": "IdentifierMediaDRMID",
                    "identifier_meid": "IdentifierMEID",
                    "identifier_phone_number": "IdentifierPhoneNumber",
                    "identifier_serial": "IdentifierSerial",
                    "identifier_wifi_mac": "IdentifierWifiMAC",
                    "serial_number": "SerialNumber",
                },
                "name": "name",
                "proxy": {
                    "host": "host",
                    "password": "password",
                    "port": 0,
                    "user": "user",
                },
                "smart_ip": True,
            },
        )
        assert_matches_type(Profile, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.profiles.with_raw_response.update(
            profile_id="profileId",
            name="x",
            spec={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        profile = await response.parse()
        assert_matches_type(Profile, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncMobilerun) -> None:
        async with async_client.profiles.with_streaming_response.update(
            profile_id="profileId",
            name="x",
            spec={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            profile = await response.parse()
            assert_matches_type(Profile, profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `profile_id` but received ''"):
            await async_client.profiles.with_raw_response.update(
                profile_id="",
                name="x",
                spec={},
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncMobilerun) -> None:
        profile = await async_client.profiles.list()
        assert_matches_type(ProfileListResponse, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncMobilerun) -> None:
        profile = await async_client.profiles.list(
            name="name",
            order_by="name",
            order_by_direction="asc",
            page=0,
            page_size=0,
        )
        assert_matches_type(ProfileListResponse, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.profiles.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        profile = await response.parse()
        assert_matches_type(ProfileListResponse, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncMobilerun) -> None:
        async with async_client.profiles.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            profile = await response.parse()
            assert_matches_type(ProfileListResponse, profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncMobilerun) -> None:
        profile = await async_client.profiles.delete(
            "profileId",
        )
        assert_matches_type(ProfileDeleteResponse, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.profiles.with_raw_response.delete(
            "profileId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        profile = await response.parse()
        assert_matches_type(ProfileDeleteResponse, profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncMobilerun) -> None:
        async with async_client.profiles.with_streaming_response.delete(
            "profileId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            profile = await response.parse()
            assert_matches_type(ProfileDeleteResponse, profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `profile_id` but received ''"):
            await async_client.profiles.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .task import Task
from .._models import BaseModel

__all__ = ["TaskRetrieveResponse"]


class TaskRetrieveResponse(BaseModel):
    task: Task
    """The task"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["PaginationMeta"]


class PaginationMeta(BaseModel):
    """Pagination metadata."""

    has_next: bool = FieldInfo(alias="hasNext")
    """Whether there is a next page"""

    has_prev: bool = FieldInfo(alias="hasPrev")
    """Whether there is a previous page"""

    page: int
    """Current page number (1-based)"""

    pages: int
    """Total number of pages"""

    page_size: int = FieldInfo(alias="pageSize")
    """Number of items per page"""

    total: int
    """Total number of items"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["HookUpdateResponse"]


class HookUpdateResponse(BaseModel):
    """Response model after successfully editing a hook."""

    id: str
    """The subscription ID"""

    state: str
    """The hook state"""

    updated: bool
    """Whether the hook was updated"""

    url: str
    """The webhook URL"""

    events: Optional[List[str]] = None
    """List of subscribed events"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["HookUnsubscribeResponse"]


class HookUnsubscribeResponse(BaseModel):
    """Response model after successful unsubscription."""

    id: str
    """The subscription ID"""

    unsubscribed: bool
    """Whether unsubscription was successful"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

from .._types import SequenceNotStr

__all__ = ["HookUpdateParams"]


class HookUpdateParams(TypedDict, total=False):
    events: Optional[SequenceNotStr[str]]
    """Updated list of events to subscribe to"""

    state: Optional[str]
    """Updated hook state (active, disabled, deleted)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["DeviceIdentifiers"]


class DeviceIdentifiers(BaseModel):
    bootloader_serial_number: str = FieldInfo(alias="BootloaderSerialNumber")

    identifier_android_id: str = FieldInfo(alias="IdentifierAndroidID")

    identifier_app_set_id: str = FieldInfo(alias="IdentifierAppSetID")

    identifier_bluetooth_mac: str = FieldInfo(alias="IdentifierBluetoothMAC")

    identifier_gaid: str = FieldInfo(alias="IdentifierGAID")

    identifier_gsfid: str = FieldInfo(alias="IdentifierGSFID")

    identifier_iccid: str = FieldInfo(alias="IdentifierICCID")

    identifier_imei: str = FieldInfo(alias="IdentifierIMEI")

    identifier_imsi: str = FieldInfo(alias="IdentifierIMSI")

    identifier_media_drmid: str = FieldInfo(alias="IdentifierMediaDRMID")

    identifier_meid: str = FieldInfo(alias="IdentifierMEID")

    identifier_phone_number: str = FieldInfo(alias="IdentifierPhoneNumber")

    identifier_serial: str = FieldInfo(alias="IdentifierSerial")

    identifier_wifi_mac: str = FieldInfo(alias="IdentifierWifiMAC")

    serial_number: str = FieldInfo(alias="SerialNumber")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .usage_result import UsageResult

__all__ = [
    "TaskGetTrajectoryResponse",
    "Trajectory",
    "TrajectoryTrajectoryCreatedEvent",
    "TrajectoryTrajectoryCreatedEventData",
    "TrajectoryTrajectoryExceptionEvent",
    "TrajectoryTrajectoryExceptionEventData",
    "TrajectoryTrajectoryCancelEvent",
    "TrajectoryTrajectoryCancelEventData",
    "TrajectoryTrajectoryScreenshotEvent",
    "TrajectoryTrajectoryScreenshotEventData",
    "TrajectoryTrajectoryStartEvent",
    "TrajectoryTrajectoryFinalizeEvent",
    "TrajectoryTrajectoryFinalizeEventData",
    "TrajectoryTrajectoryStopEvent",
    "TrajectoryTrajectoryResultEvent",
    "TrajectoryTrajectoryResultEventData",
    "TrajectoryTrajectoryManagerInputEvent",
    "TrajectoryTrajectoryManagerPlanEvent",
    "TrajectoryTrajectoryManagerPlanEventData",
    "TrajectoryTrajectoryExecutorInputEvent",
    "TrajectoryTrajectoryExecutorInputEventData",
    "TrajectoryTrajectoryExecutorResultEvent",
    "TrajectoryTrajectoryExecutorResultEventData",
    "TrajectoryTrajectoryFastAgentInputEvent",
    "TrajectoryTrajectoryFastAgentResponseEvent",
    "TrajectoryTrajectoryFastAgentResponseEventData",
    "TrajectoryTrajectoryFastAgentToolCallEvent",
    "TrajectoryTrajectoryFastAgentToolCallEventData",
    "TrajectoryTrajectoryFastAgentOutputEvent",
    "TrajectoryTrajectoryFastAgentOutputEventData",
    "TrajectoryTrajectoryFastAgentEndEvent",
    "TrajectoryTrajectoryFastAgentEndEventData",
    "TrajectoryTrajectoryFastAgentExecuteEvent",
    "TrajectoryTrajectoryFastAgentExecuteEventData",
    "TrajectoryTrajectoryFastAgentResultEvent",
    "TrajectoryTrajectoryFastAgentResultEventData",
    "TrajectoryTrajectoryToolExecutionEvent",
    "TrajectoryTrajectoryToolExecutionEventData",
    "TrajectoryTrajectoryRecordUiStateEvent",
    "TrajectoryTrajectoryRecordUiStateEventData",
    "TrajectoryTrajectoryManagerContextEvent",
    "TrajectoryTrajectoryManagerResponseEvent",
    "TrajectoryTrajectoryManagerResponseEventData",
    "TrajectoryTrajectoryManagerPlanDetailsEvent",
    "TrajectoryTrajectoryManagerPlanDetailsEventData",
    "TrajectoryTrajectoryExecutorContextEvent",
    "TrajectoryTrajectoryExecutorContextEventData",
    "TrajectoryTrajectoryExecutorResponseEvent",
    "TrajectoryTrajectoryExecutorResponseEventData",
    "TrajectoryTrajectoryExecutorActionEvent",
    "TrajectoryTrajectoryExecutorActionEventData",
    "TrajectoryTrajectoryExecutorActionResultEvent",
    "TrajectoryTrajectoryExecutorActionResultEventData",
    "TrajectoryTrajectoryUserMessageEvent",
    "TrajectoryTrajectoryUserMessageEventData",
    "TrajectoryTrajectoryUnknownEvent",
]


class TrajectoryTrajectoryCreatedEventData(BaseModel):
    id: str

    stream_url: str = FieldInfo(alias="streamUrl")


class TrajectoryTrajectoryCreatedEvent(BaseModel):
    data: TrajectoryTrajectoryCreatedEventData

    event: Literal["CreatedEvent"]


class TrajectoryTrajectoryExceptionEventData(BaseModel):
    exception: str


class TrajectoryTrajectoryExceptionEvent(BaseModel):
    data: TrajectoryTrajectoryExceptionEventData

    event: Literal["ExceptionEvent"]


class TrajectoryTrajectoryCancelEventData(BaseModel):
    reason: str


class TrajectoryTrajectoryCancelEvent(BaseModel):
    data: TrajectoryTrajectoryCancelEventData

    event: Literal["CancelEvent"]


class TrajectoryTrajectoryScreenshotEventData(BaseModel):
    index: int

    url: str


class TrajectoryTrajectoryScreenshotEvent(BaseModel):
    data: TrajectoryTrajectoryScreenshotEventData

    event: Literal["ScreenshotEvent"]


class TrajectoryTrajectoryStartEvent(BaseModel):
    data: object
    """Implicit entry event sent to kick off a `Workflow.run()`."""

    event: Literal["StartEvent"]


class TrajectoryTrajectoryFinalizeEventData(BaseModel):
    """Trigger finalization."""

    reason: str

    success: bool


class TrajectoryTrajectoryFinalizeEvent(BaseModel):
    data: TrajectoryTrajectoryFinalizeEventData
    """Trigger finalization."""

    event: Literal["FinalizeEvent"]


class TrajectoryTrajectoryStopEvent(BaseModel):
    data: object
    """Terminal event that signals the workflow has completed.

    The `result` property contains the return value of the workflow run. When a
    custom stop event subclass is used, the workflow result is that event instance
    itself.

    Examples:
    `python # default stop event: result holds the value return StopEvent(result={"answer": 42}) `

        Subclassing to provide a custom result:

        ```python
        class MyStopEv(StopEvent):
            pass

        @step
        async def my_step(self, ctx: Context, ev: StartEvent) -> MyStopEv:
            return MyStopEv(result={"answer": 42})
    """

    event: Literal["StopEvent"]


class TrajectoryTrajectoryResultEventData(BaseModel):
    """Lazy wrapper — avoids importing droidrun at module level.

    The worker uses droidrun's ResultEvent directly; this model only
    exists so the API OpenAPI schema can reference it without the heavy
    droidrun import.
    """

    message: Optional[str] = None

    steps: Optional[int] = None

    structured_output: Optional[Dict[str, object]] = None

    success: Optional[bool] = None


class TrajectoryTrajectoryResultEvent(BaseModel):
    data: TrajectoryTrajectoryResultEventData
    """Lazy wrapper — avoids importing droidrun at module level.

    The worker uses droidrun's ResultEvent directly; this model only exists so the
    API OpenAPI schema can reference it without the heavy droidrun import.
    """

    event: Literal["ResultEvent"]


class TrajectoryTrajectoryManagerInputEvent(BaseModel):
    data: object
    """Trigger Manager workflow for planning"""

    event: Literal["ManagerInputEvent"]


class TrajectoryTrajectoryManagerPlanEventData(BaseModel):
    """Coordination event from ManagerAgent to DroidAgent.

    Used for workflow step routing only (NOT streamed to frontend).
    For internal events with memory_update metadata, see ManagerPlanDetailsEvent.
    """

    current_subgoal: str

    plan: str

    thought: str

    answer: Optional[str] = None

    success: Optional[bool] = None


class TrajectoryTrajectoryManagerPlanEvent(BaseModel):
    data: TrajectoryTrajectoryManagerPlanEventData
    """Coordination event from ManagerAgent to DroidAgent.

    Used for workflow step routing only (NOT streamed to frontend). For internal
    events with memory_update metadata, see ManagerPlanDetailsEvent.
    """

    event: Literal["ManagerPlanEvent"]


class TrajectoryTrajectoryExecutorInputEventData(BaseModel):
    """Trigger Executor workflow for action execution"""

    current_subgoal: str


class TrajectoryTrajectoryExecutorInputEvent(BaseModel):
    data: TrajectoryTrajectoryExecutorInputEventData
    """Trigger Executor workflow for action execution"""

    event: Literal["ExecutorInputEvent"]


class TrajectoryTrajectoryExecutorResultEventData(BaseModel):
    """Executor finished with action result."""

    action: Dict[str, object]

    error: str

    outcome: bool

    summary: str


class TrajectoryTrajectoryExecutorResultEvent(BaseModel):
    data: TrajectoryTrajectoryExecutorResultEventData
    """Executor finished with action result."""

    event: Literal["ExecutorResultEvent"]


class TrajectoryTrajectoryFastAgentInputEvent(BaseModel):
    data: object
    """Input ready for LLM."""

    event: Literal["FastAgentInputEvent"]


class TrajectoryTrajectoryFastAgentResponseEventData(BaseModel):
    """LLM response received."""

    thought: str

    code: Optional[str] = None

    usage: Optional[UsageResult] = None


class TrajectoryTrajectoryFastAgentResponseEvent(BaseModel):
    data: TrajectoryTrajectoryFastAgentResponseEventData
    """LLM response received."""

    event: Literal["FastAgentResponseEvent"]


class TrajectoryTrajectoryFastAgentToolCallEventData(BaseModel):
    """Tool calls ready to execute."""

    tool_calls_repr: str


class TrajectoryTrajectoryFastAgentToolCallEvent(BaseModel):
    data: TrajectoryTrajectoryFastAgentToolCallEventData
    """Tool calls ready to execute."""

    event: Literal["FastAgentToolCallEvent"]


class TrajectoryTrajectoryFastAgentOutputEventData(BaseModel):
    """Tool execution result."""

    output: str


class TrajectoryTrajectoryFastAgentOutputEvent(BaseModel):
    data: TrajectoryTrajectoryFastAgentOutputEventData
    """Tool execution result."""

    event: Literal["FastAgentOutputEvent"]


class TrajectoryTrajectoryFastAgentEndEventData(BaseModel):
    """FastAgent finished."""

    reason: str

    success: bool

    tool_call_count: Optional[int] = None


class TrajectoryTrajectoryFastAgentEndEvent(BaseModel):
    data: TrajectoryTrajectoryFastAgentEndEventData
    """FastAgent finished."""

    event: Literal["FastAgentEndEvent"]


class TrajectoryTrajectoryFastAgentExecuteEventData(BaseModel):
    instruction: str


class TrajectoryTrajectoryFastAgentExecuteEvent(BaseModel):
    data: TrajectoryTrajectoryFastAgentExecuteEventData

    event: Literal["FastAgentExecuteEvent"]


class TrajectoryTrajectoryFastAgentResultEventData(BaseModel):
    instruction: str

    reason: str

    success: bool


class TrajectoryTrajectoryFastAgentResultEvent(BaseModel):
    data: TrajectoryTrajectoryFastAgentResultEventData

    event: Literal["FastAgentResultEvent"]


class TrajectoryTrajectoryToolExecutionEventData(BaseModel):
    """Emitted after every tool call dispatched through ToolRegistry."""

    success: bool

    summary: str

    tool_args: Dict[str, object]

    tool_name: str


class TrajectoryTrajectoryToolExecutionEvent(BaseModel):
    data: TrajectoryTrajectoryToolExecutionEventData
    """Emitted after every tool call dispatched through ToolRegistry."""

    event: Literal["ToolExecutionEvent"]


class TrajectoryTrajectoryRecordUiStateEventData(BaseModel):
    index: int

    url: str


class TrajectoryTrajectoryRecordUiStateEvent(BaseModel):
    data: TrajectoryTrajectoryRecordUiStateEventData

    event: Literal["RecordUIStateEvent"]


class TrajectoryTrajectoryManagerContextEvent(BaseModel):
    data: object
    """Context prepared, ready for LLM call."""

    event: Literal["ManagerContextEvent"]


class TrajectoryTrajectoryManagerResponseEventData(BaseModel):
    """LLM response received, ready for parsing."""

    response: str

    usage: Optional[UsageResult] = None


class TrajectoryTrajectoryManagerResponseEvent(BaseModel):
    data: TrajectoryTrajectoryManagerResponseEventData
    """LLM response received, ready for parsing."""

    event: Literal["ManagerResponseEvent"]


class TrajectoryTrajectoryManagerPlanDetailsEventData(BaseModel):
    """Plan parsed and ready (internal event with full details)."""

    plan: str

    subgoal: str

    thought: str

    answer: Optional[str] = None

    full_response: Optional[str] = None

    memory_update: Optional[str] = None

    progress_summary: Optional[str] = None

    success: Optional[bool] = None


class TrajectoryTrajectoryManagerPlanDetailsEvent(BaseModel):
    data: TrajectoryTrajectoryManagerPlanDetailsEventData
    """Plan parsed and ready (internal event with full details)."""

    event: Literal["ManagerPlanDetailsEvent"]


class TrajectoryTrajectoryExecutorContextEventData(BaseModel):
    """Context prepared, ready for LLM call."""

    subgoal: str


class TrajectoryTrajectoryExecutorContextEvent(BaseModel):
    data: TrajectoryTrajectoryExecutorContextEventData
    """Context prepared, ready for LLM call."""

    event: Literal["ExecutorContextEvent"]


class TrajectoryTrajectoryExecutorResponseEventData(BaseModel):
    """LLM response received, ready for parsing."""

    response: str

    usage: Optional[UsageResult] = None


class TrajectoryTrajectoryExecutorResponseEvent(BaseModel):
    data: TrajectoryTrajectoryExecutorResponseEventData
    """LLM response received, ready for parsing."""

    event: Literal["ExecutorResponseEvent"]


class TrajectoryTrajectoryExecutorActionEventData(BaseModel):
    """Action parsed, ready to execute."""

    action_json: str

    description: str

    thought: str

    full_response: Optional[str] = None


class TrajectoryTrajectoryExecutorActionEvent(BaseModel):
    data: TrajectoryTrajectoryExecutorActionEventData
    """Action parsed, ready to execute."""

    event: Literal["ExecutorActionEvent"]


class TrajectoryTrajectoryExecutorActionResultEventData(BaseModel):
    """Action execution result (internal event with full details)."""

    action: Dict[str, object]

    error: str

    success: bool

    summary: str

    full_response: Optional[str] = None

    thought: Optional[str] = None


class TrajectoryTrajectoryExecutorActionResultEvent(BaseModel):
    data: TrajectoryTrajectoryExecutorActionResultEventData
    """Action execution result (internal event with full details)."""

    event: Literal["ExecutorActionResultEvent"]


class TrajectoryTrajectoryUserMessageEventData(BaseModel):
    """Tracks the lifecycle of an external user message: queued → applied | dropped."""

    action: str

    message_ids: List[str]

    consumer: Optional[str] = None

    reason: Optional[str] = None

    step_number: Optional[int] = None


class TrajectoryTrajectoryUserMessageEvent(BaseModel):
    data: TrajectoryTrajectoryUserMessageEventData
    """Tracks the lifecycle of an external user message: queued → applied | dropped."""

    event: Literal["UserMessageEvent"]


class TrajectoryTrajectoryUnknownEvent(BaseModel):
    event: str

    data: Optional[Dict[str, object]] = None


Trajectory: TypeAlias = Union[
    TrajectoryTrajectoryCreatedEvent,
    TrajectoryTrajectoryExceptionEvent,
    TrajectoryTrajectoryCancelEvent,
    TrajectoryTrajectoryScreenshotEvent,
    TrajectoryTrajectoryStartEvent,
    TrajectoryTrajectoryFinalizeEvent,
    TrajectoryTrajectoryStopEvent,
    TrajectoryTrajectoryResultEvent,
    TrajectoryTrajectoryManagerInputEvent,
    TrajectoryTrajectoryManagerPlanEvent,
    TrajectoryTrajectoryExecutorInputEvent,
    TrajectoryTrajectoryExecutorResultEvent,
    TrajectoryTrajectoryFastAgentInputEvent,
    TrajectoryTrajectoryFastAgentResponseEvent,
    TrajectoryTrajectoryFastAgentToolCallEvent,
    TrajectoryTrajectoryFastAgentOutputEvent,
    TrajectoryTrajectoryFastAgentEndEvent,
    TrajectoryTrajectoryFastAgentExecuteEvent,
    TrajectoryTrajectoryFastAgentResultEvent,
    TrajectoryTrajectoryToolExecutionEvent,
    TrajectoryTrajectoryRecordUiStateEvent,
    TrajectoryTrajectoryManagerContextEvent,
    TrajectoryTrajectoryManagerResponseEvent,
    TrajectoryTrajectoryManagerPlanDetailsEvent,
    TrajectoryTrajectoryExecutorContextEvent,
    TrajectoryTrajectoryExecutorResponseEvent,
    TrajectoryTrajectoryExecutorActionEvent,
    TrajectoryTrajectoryExecutorActionResultEvent,
    TrajectoryTrajectoryUserMessageEvent,
    TrajectoryTrajectoryUnknownEvent,
]


class TaskGetTrajectoryResponse(BaseModel):
    trajectory: List[Trajectory]
    """The trajectory of the task"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from .file_info import FileInfo

__all__ = ["FileListResponse"]


class FileListResponse(BaseModel):
    files: Optional[List[FileInfo]] = None

    path: str

    total: int

    schema_: Optional[str] = FieldInfo(alias="$schema", default=None)
    """A URL to the JSON Schema for this object."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel
from .credential import Credential

__all__ = ["CredentialRetrieveResponse"]


class CredentialRetrieveResponse(BaseModel):
    data: Credential



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .rect import Rect as Rect
from .file_info import FileInfo as FileInfo
from .app_list_params import AppListParams as AppListParams
from .state_ui_params import StateUiParams as StateUiParams
from .app_start_params import AppStartParams as AppStartParams
from .file_list_params import FileListParams as FileListParams
from .task_list_params import TaskListParams as TaskListParams
from .action_tap_params import ActionTapParams as ActionTapParams
from .app_list_response import AppListResponse as AppListResponse
from .state_ui_response import StateUiResponse as StateUiResponse
from .app_install_params import AppInstallParams as AppInstallParams
from .file_delete_params import FileDeleteParams as FileDeleteParams
from .file_list_response import FileListResponse as FileListResponse
from .file_upload_params import FileUploadParams as FileUploadParams
from .task_list_response import TaskListResponse as TaskListResponse
from .time_time_response import TimeTimeResponse as TimeTimeResponse
from .action_swipe_params import ActionSwipeParams as ActionSwipeParams
from .keyboard_key_params import KeyboardKeyParams as KeyboardKeyParams
from .location_set_params import LocationSetParams as LocationSetParams
from .package_list_params import PackageListParams as PackageListParams
from .action_global_params import ActionGlobalParams as ActionGlobalParams
from .file_download_params import FileDownloadParams as FileDownloadParams
from .proxy_connect_params import ProxyConnectParams as ProxyConnectParams
from .time_set_time_params import TimeSetTimeParams as TimeSetTimeParams
from .keyboard_write_params import KeyboardWriteParams as KeyboardWriteParams
from .location_get_response import LocationGetResponse as LocationGetResponse
from .package_list_response import PackageListResponse as PackageListResponse
from .profile_update_params import ProfileUpdateParams as ProfileUpdateParams
from .file_download_response import FileDownloadResponse as FileDownloadResponse
from .time_timezone_response import TimeTimezoneResponse as TimeTimezoneResponse
from .state_screenshot_params import StateScreenshotParams as StateScreenshotParams
from .time_set_timezone_params import TimeSetTimezoneParams as TimeSetTimezoneParams
from .state_screenshot_response import StateScreenshotResponse as StateScreenshotResponse
from .action_overlay_visible_response import ActionOverlayVisibleResponse as ActionOverlayVisibleResponse
from .action_set_overlay_visible_params import ActionSetOverlayVisibleParams as ActionSetOverlayVisibleParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ProxyConnectParams", "Proxy"]


class ProxyConnectParams(TypedDict, total=False):
    host: str

    password: str

    port: int

    proxy: Proxy
    """Preferred new format."""

    smart_ip: Annotated[bool, PropertyInfo(alias="smartIp")]

    user: str

    x_device_display_id: Annotated[int, PropertyInfo(alias="X-Device-Display-ID")]


class Proxy(TypedDict, total=False):
    """Preferred new format."""

    host: Required[str]

    password: Required[str]

    port: Required[int]

    user: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["DeviceIdentifiers"]


class DeviceIdentifiers(TypedDict, total=False):
    bootloader_serial_number: Required[Annotated[str, PropertyInfo(alias="BootloaderSerialNumber")]]

    identifier_android_id: Required[Annotated[str, PropertyInfo(alias="IdentifierAndroidID")]]

    identifier_app_set_id: Required[Annotated[str, PropertyInfo(alias="IdentifierAppSetID")]]

    identifier_bluetooth_mac: Required[Annotated[str, PropertyInfo(alias="IdentifierBluetoothMAC")]]

    identifier_gaid: Required[Annotated[str, PropertyInfo(alias="IdentifierGAID")]]

    identifier_gsfid: Required[Annotated[str, PropertyInfo(alias="IdentifierGSFID")]]

    identifier_iccid: Required[Annotated[str, PropertyInfo(alias="IdentifierICCID")]]

    identifier_imei: Required[Annotated[str, PropertyInfo(alias="IdentifierIMEI")]]

    identifier_imsi: Required[Annotated[str, PropertyInfo(alias="IdentifierIMSI")]]

    identifier_media_drmid: Required[Annotated[str, PropertyInfo(alias="IdentifierMediaDRMID")]]

    identifier_meid: Required[Annotated[str, PropertyInfo(alias="IdentifierMEID")]]

    identifier_phone_number: Required[Annotated[str, PropertyInfo(alias="IdentifierPhoneNumber")]]

    identifier_serial: Required[Annotated[str, PropertyInfo(alias="IdentifierSerial")]]

    identifier_wifi_mac: Required[Annotated[str, PropertyInfo(alias="IdentifierWifiMAC")]]

    serial_number: Required[Annotated[str, PropertyInfo(alias="SerialNumber")]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["CarrierListParams"]


class CarrierListParams(TypedDict, total=False):
    country: str
    """Filter by country name"""

    country_iso: Annotated[str, PropertyInfo(alias="countryISO")]
    """Filter by country ISO code"""

    order_by: Annotated[
        Literal["id", "mcc", "mnc", "operator", "country", "country_iso"], PropertyInfo(alias="orderBy")
    ]
    """Field to order by"""

    order_dir: Annotated[Literal["asc", "desc"], PropertyInfo(alias="orderDir")]
    """Order direction"""

    page: int
    """Page number"""

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Items per page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["AppListResponse", "AppListResponseItem"]


class AppListResponseItem(BaseModel):
    is_system_app: bool = FieldInfo(alias="isSystemApp")

    label: str

    package_name: str = FieldInfo(alias="packageName")

    version_code: int = FieldInfo(alias="versionCode")

    version_name: str = FieldInfo(alias="versionName")


AppListResponse: TypeAlias = List[AppListResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .config import Config as Config
from .device_spec import DeviceSpec as DeviceSpec
from .device_carrier import DeviceCarrier as DeviceCarrier
from .device_identifiers import DeviceIdentifiers as DeviceIdentifiers



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from mobilerun import Mobilerun, AsyncMobilerun
from tests.utils import assert_matches_type
from mobilerun.types import (
    Device,
    DeviceListResponse,
    DeviceCountResponse,
)
from mobilerun._utils import parse_datetime

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDevices:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Mobilerun) -> None:
        device = client.devices.create()
        assert_matches_type(Device, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Mobilerun) -> None:
        device = client.devices.create(
            device_type="dedicated_physical_device",
            apps=["string"],
            carrier={
                "gsm_operator_alpha": "GsmOperatorAlpha",
                "gsm_operator_numeric": 0,
                "gsm_sim_operator_alpha": "GsmSimOperatorAlpha",
                "gsm_sim_operator_iso_country": "GsmSimOperatorIsoCountry",
                "gsm_sim_operator_numeric": 0,
                "persist_sys_timezone": "PersistSysTimezone",
            },
            files=["string"],
            identifiers={
                "bootloader_serial_number": "BootloaderSerialNumber",
                "identifier_android_id": "IdentifierAndroidID",
                "identifier_app_set_id": "IdentifierAppSetID",
                "identifier_bluetooth_mac": "IdentifierBluetoothMAC",
                "identifier_gaid": "IdentifierGAID",
                "identifier_gsfid": "IdentifierGSFID",
                "identifier_iccid": "IdentifierICCID",
                "identifier_imei": "IdentifierIMEI",
                "identifier_imsi": "IdentifierIMSI",
                "identifier_media_drmid": "IdentifierMediaDRMID",
                "identifier_meid": "IdentifierMEID",
                "identifier_phone_number": "IdentifierPhoneNumber",
                "identifier_serial": "IdentifierSerial",
                "identifier_wifi_mac": "IdentifierWifiMAC",
                "serial_number": "SerialNumber",
            },
            name="name",
            proxy={
                "host": "host",
                "password": "password",
                "port": 0,
                "user": "user",
            },
            smart_ip=True,
        )
        assert_matches_type(Device, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Mobilerun) -> None:
        response = client.devices.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        device = response.parse()
        assert_matches_type(Device, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Mobilerun) -> None:
        with client.devices.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            device = response.parse()
            assert_matches_type(Device, device, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Mobilerun) -> None:
        device = client.devices.retrieve(
            "deviceId",
        )
        assert_matches_type(Device, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Mobilerun) -> None:
        response = client.devices.with_raw_response.retrieve(
            "deviceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        device = response.parse()
        assert_matches_type(Device, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Mobilerun) -> None:
        with client.devices.with_streaming_response.retrieve(
            "deviceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            device = response.parse()
            assert_matches_type(Device, device, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            client.devices.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Mobilerun) -> None:
        device = client.devices.list()
        assert_matches_type(DeviceListResponse, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Mobilerun) -> None:
        device = client.devices.list(
            country="country",
            name="name",
            order_by="id",
            order_by_direction="asc",
            page=0,
            page_size=0,
            state=["creating"],
            type="dedicated_physical_device",
        )
        assert_matches_type(DeviceListResponse, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Mobilerun) -> None:
        response = client.devices.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        device = response.parse()
        assert_matches_type(DeviceListResponse, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Mobilerun) -> None:
        with client.devices.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            device = response.parse()
            assert_matches_type(DeviceListResponse, device, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_count(self, client: Mobilerun) -> None:
        device = client.devices.count()
        assert_matches_type(DeviceCountResponse, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_count(self, client: Mobilerun) -> None:
        response = client.devices.with_raw_response.count()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        device = response.parse()
        assert_matches_type(DeviceCountResponse, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_count(self, client: Mobilerun) -> None:
        with client.devices.with_streaming_response.count() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            device = response.parse()
            assert_matches_type(DeviceCountResponse, device, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_terminate(self, client: Mobilerun) -> None:
        device = client.devices.terminate(
            device_id="deviceId",
        )
        assert device is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_terminate_with_all_params(self, client: Mobilerun) -> None:
        device = client.devices.terminate(
            device_id="deviceId",
            previous_device_id="previousDeviceId",
            terminate_at=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert device is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_terminate(self, client: Mobilerun) -> None:
        response = client.devices.with_raw_response.terminate(
            device_id="deviceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        device = response.parse()
        assert device is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_terminate(self, client: Mobilerun) -> None:
        with client.devices.with_streaming_response.terminate(
            device_id="deviceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            device = response.parse()
            assert device is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_terminate(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            client.devices.with_raw_response.terminate(
                device_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_wait_ready(self, client: Mobilerun) -> None:
        device = client.devices.wait_ready(
            "deviceId",
        )
        assert_matches_type(Device, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_wait_ready(self, client: Mobilerun) -> None:
        response = client.devices.with_raw_response.wait_ready(
            "deviceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        device = response.parse()
        assert_matches_type(Device, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_wait_ready(self, client: Mobilerun) -> None:
        with client.devices.with_streaming_response.wait_ready(
            "deviceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            device = response.parse()
            assert_matches_type(Device, device, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_wait_ready(self, client: Mobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            client.devices.with_raw_response.wait_ready(
                "",
            )


class TestAsyncDevices:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncMobilerun) -> None:
        device = await async_client.devices.create()
        assert_matches_type(Device, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncMobilerun) -> None:
        device = await async_client.devices.create(
            device_type="dedicated_physical_device",
            apps=["string"],
            carrier={
                "gsm_operator_alpha": "GsmOperatorAlpha",
                "gsm_operator_numeric": 0,
                "gsm_sim_operator_alpha": "GsmSimOperatorAlpha",
                "gsm_sim_operator_iso_country": "GsmSimOperatorIsoCountry",
                "gsm_sim_operator_numeric": 0,
                "persist_sys_timezone": "PersistSysTimezone",
            },
            files=["string"],
            identifiers={
                "bootloader_serial_number": "BootloaderSerialNumber",
                "identifier_android_id": "IdentifierAndroidID",
                "identifier_app_set_id": "IdentifierAppSetID",
                "identifier_bluetooth_mac": "IdentifierBluetoothMAC",
                "identifier_gaid": "IdentifierGAID",
                "identifier_gsfid": "IdentifierGSFID",
                "identifier_iccid": "IdentifierICCID",
                "identifier_imei": "IdentifierIMEI",
                "identifier_imsi": "IdentifierIMSI",
                "identifier_media_drmid": "IdentifierMediaDRMID",
                "identifier_meid": "IdentifierMEID",
                "identifier_phone_number": "IdentifierPhoneNumber",
                "identifier_serial": "IdentifierSerial",
                "identifier_wifi_mac": "IdentifierWifiMAC",
                "serial_number": "SerialNumber",
            },
            name="name",
            proxy={
                "host": "host",
                "password": "password",
                "port": 0,
                "user": "user",
            },
            smart_ip=True,
        )
        assert_matches_type(Device, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.devices.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        device = await response.parse()
        assert_matches_type(Device, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncMobilerun) -> None:
        async with async_client.devices.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            device = await response.parse()
            assert_matches_type(Device, device, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncMobilerun) -> None:
        device = await async_client.devices.retrieve(
            "deviceId",
        )
        assert_matches_type(Device, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.devices.with_raw_response.retrieve(
            "deviceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        device = await response.parse()
        assert_matches_type(Device, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncMobilerun) -> None:
        async with async_client.devices.with_streaming_response.retrieve(
            "deviceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            device = await response.parse()
            assert_matches_type(Device, device, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            await async_client.devices.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncMobilerun) -> None:
        device = await async_client.devices.list()
        assert_matches_type(DeviceListResponse, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncMobilerun) -> None:
        device = await async_client.devices.list(
            country="country",
            name="name",
            order_by="id",
            order_by_direction="asc",
            page=0,
            page_size=0,
            state=["creating"],
            type="dedicated_physical_device",
        )
        assert_matches_type(DeviceListResponse, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.devices.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        device = await response.parse()
        assert_matches_type(DeviceListResponse, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncMobilerun) -> None:
        async with async_client.devices.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            device = await response.parse()
            assert_matches_type(DeviceListResponse, device, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_count(self, async_client: AsyncMobilerun) -> None:
        device = await async_client.devices.count()
        assert_matches_type(DeviceCountResponse, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_count(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.devices.with_raw_response.count()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        device = await response.parse()
        assert_matches_type(DeviceCountResponse, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_count(self, async_client: AsyncMobilerun) -> None:
        async with async_client.devices.with_streaming_response.count() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            device = await response.parse()
            assert_matches_type(DeviceCountResponse, device, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_terminate(self, async_client: AsyncMobilerun) -> None:
        device = await async_client.devices.terminate(
            device_id="deviceId",
        )
        assert device is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_terminate_with_all_params(self, async_client: AsyncMobilerun) -> None:
        device = await async_client.devices.terminate(
            device_id="deviceId",
            previous_device_id="previousDeviceId",
            terminate_at=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert device is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_terminate(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.devices.with_raw_response.terminate(
            device_id="deviceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        device = await response.parse()
        assert device is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_terminate(self, async_client: AsyncMobilerun) -> None:
        async with async_client.devices.with_streaming_response.terminate(
            device_id="deviceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            device = await response.parse()
            assert device is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_terminate(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            await async_client.devices.with_raw_response.terminate(
                device_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_wait_ready(self, async_client: AsyncMobilerun) -> None:
        device = await async_client.devices.wait_ready(
            "deviceId",
        )
        assert_matches_type(Device, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_wait_ready(self, async_client: AsyncMobilerun) -> None:
        response = await async_client.devices.with_raw_response.wait_ready(
            "deviceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        device = await response.parse()
        assert_matches_type(Device, device, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_wait_ready(self, async_client: AsyncMobilerun) -> None:
        async with async_client.devices.with_streaming_response.wait_ready(
            "deviceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            device = await response.parse()
            assert_matches_type(Device, device, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_wait_ready(self, async_client: AsyncMobilerun) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            await async_client.devices.with_raw_response.wait_ready(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo
from .shared_params.config import Config
from .shared_params.device_carrier import DeviceCarrier
from .shared_params.device_identifiers import DeviceIdentifiers

__all__ = ["DeviceCreateParams"]


class DeviceCreateParams(TypedDict, total=False):
    device_type: Annotated[
        Literal["dedicated_physical_device", "dedicated_premium_device", "dedicated_emulated_device"],
        PropertyInfo(alias="deviceType"),
    ]

    apps: Optional[SequenceNotStr[str]]

    carrier: DeviceCarrier

    files: Optional[SequenceNotStr[str]]

    identifiers: DeviceIdentifiers

    name: str

    proxy: Config

    smart_ip: Annotated[bool, PropertyInfo(alias="smartIp")]
